/*                               -*- Mode: C -*- 
 * 
 * ihsh_ars_iodef.c -- Functions to map between ARS entries and IODEF documents
 * 
 * Copyright (c) 2003 Arne Helme. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Arne Helme.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: ihsh_ars_iodef.c,v 1.1 2003/10/22 08:16:20 arne Exp $
 * 
 */

#include <string.h>
#include <stdlib.h>

#include <ar.h>
#include <arfree.h>
#include <arextern.h>

#include <ih/tree.h>

#include "ihsh_ars.h"
#include "ihsh_ars_iodef.h"
#include "ihsh_ars_misc.h"


/* eCSIRT.net IODEF profile supported by the driver */
#define ECSIRT_NET_IODEF_PROFILE_V10	"eCSIRT-net-IODEF-profile-v1.0"

/* **************************************************************
 * CERT: IODEF-Document field IDs
 * ************************************************************** */

/*
 * #
 * #  File exported Wed Sep 24 08:03:47 2003
 * #
 * #AR-Message-Begin                    Do Not Delete This Line
 * Schema: CERT: IODEF-Document
 * Server: weetmuts.surfnet.nl
 * Login: 
 * Password: 
 * Action: Submit
 * # Values: Submit, Query
 * Format: Short
 * # Values: Short, Full
 * 
 *                  purpose !536870946!: handling
 * # Values: handling, statistics, warning, other
 *              restriction !536870947!: need-to-know
 * # Values: public, need-to-know, private, default
 *       IODEF-Message type !536870948!: 
 * # Values: incoming, outgoing
 *                   Status !        7!: New
 * # Values: New, Processed, Archived
 *               IncidentID !        8!: VOID
 *           AlternativeIDs !536870945!: 
 *              Description !536870949!: 
 *                Submitter !        2!: $USER$
 *                 Assignee !        4!: auto-submitter
 *        Impact.completion !536870950!: succeeded
 * # Values: failed, succeeded
 *     Expectation.priority !536870955!: 
 * # Values: low, medium, high
 *  Expectation.Description !536870954!: 
 *              Impact.type !536870953!: 
 * # Values: admin, dos, file, recon, user, none,
 * #    unknown
 *                  History !       10!: 
 *                   Impact !536870951!: 
 *       Method.Description !536870952!: 
 *                     Type !536870933!: Organization
 * # Values: Organization, Person
 *                     Role !536870943!: irt
 * # Values: Creator of IODEF doc, admin, tech,
 * #    irt, cc
 *   Record.RecordData.Item !536870970!: 
 *                     name !536870917!: CERT-NL
 *                    Email !536870915!: cert-nl@surfnet.nl
 *                Telephone !536870925!: +31 302305305
 *                 Timezone !536870942!: +02:00
 * #AR-Message-End                      Do Not Delete This Line
 */

#define DOCUMENT_purpose			536870946L
#define DOCUMENT_restriction			536870947L
#define DOCUMENT_Message_type			536870948L
#define DOCUMENT_AlternativeIDs			536870945L
#define DOCUMENT_Description			536870949L
#define DOCUMENT_Impact_completion		536870950L
#define DOCUMENT_Expectation_priority		536870955L
#define DOCUMENT_Expectation_Description	536870954L
#define DOCUMENT_Impact_type			536870953L
#define DOCUMENT_Impact				536870951L
#define DOCUMENT_Method_Description		536870952L
#define DOCUMENT_Type				536870933L
#define DOCUMENT_Role				536870943L
#define DOCUMENT_Record_RecordData_Item		536870970L
#define DOCUMENT_name				536870917L
#define DOCUMENT_Email				536870915L
#define DOCUMENT_Telephone			536870925L
#define DOCUMENT_Timezone			536870942L

#define DOCUMENT_IDS 18

/* **************************************************************
 * CERT: IODEF-Document-Address field IDs
 * ************************************************************** */

/*
 * #
 * #  File exported Wed Sep 17 17:21:56 2003
 * #
 * #AR-Message-Begin                    Do Not Delete This Line
 * Schema: CERT: IODEF-Document-Address
 * Server: weetmuts.surfnet.nl
 * Login:
 * Password:
 * Action: Submit
 * # Values: Submit, Query
 * Format: Short
 * # Values: Short, Full
 * 
 *    RequestID !1!:
 *    System.category !536870914!:
 * # Values: source, target, intermediate
 *   Address.category !536870915!:
 * # Values: unknown, atm, e-mail, lotus-notes,
 * #    mac, sna, vm, ipv4-addr, ipv4-addr-hex,
 * #    ipv4-net, ipv4-net-mask, ipv6-addr, ipv6-addr-hex,
 * #    ipv6-net, ipv6-net-mask
 *            Address !536870913!:
 *          Submitter !        2!: $USER$
 *  IODEF-Document ID !536870916!:
 *  Short Description !        8!: a
 *             Status !        7!: New
 * # Values: New, Assigned, Fixed, Rejected,
 * #    Closed
 * #AR-Message-End                      Do Not Delete This Line
 */

#define ADDRESS_Address			536870913L
#define ADDRESS_System_category		536870914L
#define ADDRESS_Address_category	536870915L
#define ADDRESS_IODEF_DocumentID	536870916L

#define ADDRESS_IDS 4


/* **************************************************************
 * IHSH ARS LibIH Support local variables
 * ************************************************************** */

/* Macro to determine the size of a character array */
#define NMEMB(x) sizeof(x) / sizeof(char *)

/*
 * Enumerated types for IODEF-Document entries
 */

/* valid values for the 'purpose' attribute */
static char *purpose_strs[] = {"handling", 
			       "statistics", 
			       "warning", 
			       "other"
};

/* valid values for the 'restriction' attribute */
static char *restriction_strs[] = {"public", 
				   "need-to-know", 
				   "private", 
				   "default"
};

/* valid values for the 'completion' attribute */
static char *completion_strs[] = {"failed",
				  "completed"
};

/* valid values for the 'priority' attribute */
static char *priority_strs[] = {"low",
				"medium",
				"high"
};

/* valid values for the Impact 'type' attribute */
static char *impact_type_strs[] = {"admin",
				   "dos",
				   "file",
				   "recon",
				   "user",
				   "none"
};

/* valid values for the Contact 'type' attribute */
static char *contact_type_strs[] = {"organization",
				    "person"
};

/* valid values for the Contact 'role' attribute */
static char *contact_role_strs[] = {"Creator of IODEF doc",
				    "admin",
				    "tech",
				    "irt",
				    "cc"
};

/*
 * Enumerated types for IODEF-Document-Address entries
 */

/* valid values for the System 'category' attribute */
static char *system_category_strs[] = {"source", 
				       "target", 
				       "intermediate"
};

/* valid values for the Address 'category' attribute */
static char *address_category_strs[] = {"unknown",
					"atm",
					"e-mail",
					"lotus-notes",
					"mac",
					"sna",
					"vm",
					"ipv4-addr",
					"ipv4-addr-hex",
					"ipv4-net",
					"ipv4-net-mask",
					"ipv6-addr",
					"ipv6-addr-hex",
					"ipv6-net",
					"ipv6-net-mask"
};


/* **************************************************************
 * IHSH ARS LibIH Support local functions
 * ************************************************************** */

/* Current tree location variable */
static ih_tree_node_t *cur_node; 

/* Tree walk function */
static int tree_walk(ih_tree_t *tree, char *path)
{
    ih_err_t 	result;

    result = ih_tree_walk_by_path2(tree->t_root, path, &cur_node);
    if (result != IH_E_NONE) {
	ARS_DEBUG("tree_walk():\n\tih_tree_walk_by_path2(): %s\n\tPath %s",
		ih_err_code2str(result), path);
	return -1;
    }
    return 0;
}

/* Tree set value function */
static int tree_setval(char *value)
{
    ih_data_size_t	len;
    ih_err_t 		result;

    len = strlen(value);
    result = ih_node_set_value(cur_node, value, &len);
    if (result != IH_E_NONE) {
	ARS_DEBUG("tree_setval(): ih_node_set_value(): %s",
		ih_err_code2str(result));

	return -1;
    }
    return 0;
}

/* Tree set attribute function */
static int tree_setattr(char *attr, char *value)
{
    ih_data_size_t	len;
    ih_err_t 		result;

    len = strlen(value);
    result = ih_node_attr_set_by_name(cur_node, attr, value, &len);
    if (result != IH_E_NONE) {
	ARS_DEBUG("tree_setattr(): ih_node_attr_set_by_name(): %s",
		ih_err_code2str(result));

	return -1;
    }
    return 0;
}

/* Tree get value function */
static char *tree_getval(ih_tree_t *tree, char *path)
{
    ih_err_t		result;
    ih_asciz_t		value;
    ih_data_size_t	len;

    if (tree_walk(tree, path) == IH_E_NONE) {
	result = ih_node_get_value(cur_node, &value, &len);
	if (result != IH_E_NONE) {
	    return NULL;
	}
    }
    return (char *) value;
}

/* Tree get attribute function */
static char *tree_getattr(ih_tree_t *tree, char *path, char *attr)
{
    ih_err_t		result;
    ih_asciz_t		value;
    ih_data_size_t	len;

    if (tree_walk(tree, path) == IH_E_NONE) {
	result = ih_node_attr_get_by_name(cur_node, attr, &value, &len);
	if (result != IH_E_NONE) {
	    return NULL;
	}
    }
    return (char *) value;
}

/* Tree set attribute from enum list function */
static int tree_setattr_enum(ih_tree_t *tree, char *path, char *attr,
			     char **strs, size_t nstrs, 
			     ARFieldValueStruct *val)
{
    char *str;

    if (val->value.dataType != AR_DATA_TYPE_ENUM) {
	ARS_DEBUG("tree_setattr_enum(): dataType != AR_DATA_TYPE_ENUM");
	return -1;
    }
    /* add range test */
    if (val->value.u.enumVal < 0 || val->value.u.enumVal >= nstrs) {
	ARS_DEBUG("tree_setattr_enum(): enum out of range");
	str = "unknown";
    } else {
	str = strs[val->value.u.enumVal];
    }
    if (tree_walk(tree, path) < 0) {
	ARS_DEBUG("tree_setattr_enum(): tree_walk() failed");
	return -1;
    }
    return tree_setattr(attr, str);
}

/* Tree get attribue from enum list functions */
static int tree_getattr_enum(ih_tree_t *tree, char *path, char *attr,
			     char **strs, size_t nstrs, 
			     unsigned long fieldId,
			     ARFieldValueStruct *val)
{
    char *str;
    int num, i, ret;

    str = tree_getattr(tree, path, attr);
    if (str != NULL) {
	for (i = 0; i < nstrs; i++) {
	    if (!strcmp(str, strs[i])) {
		num = i;
		break;
	    }
	}
	if (i == nstrs) {
	    ARS_DEBUG("tree_getattr_enum(): attr: %s, illegal value: %s", 
		      attr, str);
	    exit(EXIT_FAILURE);
	    /* NOTREACHED */
	}
	val->fieldId = fieldId;
	val->value.dataType = AR_DATA_TYPE_ENUM;
	val->value.u.enumVal = (unsigned long) num;
	ret = 0;
    } else {
	ARS_DEBUG("tree_getattr_enum(): attr: %s: not found", attr);
	exit(EXIT_FAILURE);
	/* NOTREACHED */
    }
    return ret;
}

/* Tree set value from C char string function */
static int tree_setval_char(ih_tree_t *tree, char *path,  
			    ARFieldValueStruct *val)
{
    if (val->value.dataType != AR_DATA_TYPE_CHAR) {
	ARS_DEBUG("tree_setval_char(): ARS data type != CHAR");
	return -1;
    }
    if (tree_walk(tree, path) < 0) {
	ARS_DEBUG("tree_setval_char(): tree_walk() < 0");
	return -1;
    }
    return tree_setval(val->value.u.charVal);

}

/* Tree get value from C char string function */
static int tree_getval_char(ih_tree_t *tree, char *path,  
			    unsigned long fieldId,
			    ARFieldValueStruct *val)
{
    int ret;
    char *str;

    str = tree_getval(tree, path);
    if (str != NULL) {
	val->fieldId = fieldId;
	val->value.dataType = AR_DATA_TYPE_CHAR;
	val->value.u.charVal = str;
	ret = 0;
    } else {
	ARS_DEBUG("tree_getval_char(): error");
	exit(EXIT_FAILURE);
    }
    return ret;
}


/* **************************************************************
 * IHSH ARS Remedy IODEF local functions
 * ************************************************************** */

/* Function to get IODEF-Document-Address entry from ARS server */
static int getentry_iodef_address(ARControlStruct *control, ih_tree_t *tree, 
				  char *entryid)
{
    ARNameType		schema;
    AREntryIdType	entryId;
    ARStatusList     	status = {0, NULL};
    AREntryIdList	entryIdList;
    ARInternalIdList	idList;
    ARFieldValueList	fieldList;
    ARInternalId	ids[4];
    int 		i, ret;

    /* Set schema and entry ID */
    strcpy(schema, "CERT: IODEF-Document-Address");
    strncpy(entryId, entryid, AR_MAX_ENTRYID_SIZE);
    entryId[AR_MAX_ENTRYID_SIZE] = '\0'; /* to be safe */
    entryIdList.numItems = 1;
    entryIdList.entryIdList = &entryId;

    ARS_DEBUG("getentry_iodef_address():\n\tschema=%s\n\tentryId=%s",
	      schema, entryId);

    i = 0;
    ids[i++] = ADDRESS_Address;
    ids[i++] = ADDRESS_System_category;
    ids[i++] = ADDRESS_Address_category;
    ids[i++] = ADDRESS_IODEF_DocumentID;

    idList.numItems = i;
    idList.internalIdList = ids;

    /* Get entry from the ARS server */
    if (ARGetEntry(control, schema, &entryIdList, &idList, 
		   &fieldList, &status) >= AR_RETURN_ERROR) {
	ARS_DEBUG("getentry_iodef_address(): ARGetEntry failed");
	if (ars_debug == TRUE) {
	    PrintARStatusList(stderr, &status);
	}
	FreeARStatusList(&status, FALSE);
	return -1;
    }
    FreeARStatusList(&status, FALSE);

    ret = 0;
    for (i = 0; i < fieldList.numItems; i++) {
	ARFieldValueStruct *val = &fieldList.fieldValueList[i];

	if (val->value.dataType == AR_DATA_TYPE_NULL) {
	    continue; /* continue with the next value for for-loop */
	}

	ARS_DEBUG("getentry_iodef_address(): got fieldId=%d, type=%d",
		  val->fieldId,
		  val->value.dataType);
	switch (val->fieldId) {
	case ADDRESS_Address:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/IncidentData/EventData/System/Node/Address/address",
				   val);
	    break;
	case ADDRESS_System_category:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident/IncidentData/EventData/System",
				    "category", system_category_strs,
				    NMEMB(system_category_strs),
				    val);
	    break;
	case ADDRESS_Address_category:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident/IncidentData/EventData/System/Node/Address",
				    "category", address_category_strs,
				    NMEMB(address_category_strs),
				    val);
	    break;
	case ADDRESS_IODEF_DocumentID:
	    /* XXX Arne: not yet */
	    break;
	default:
	    ARS_DEBUG("getentry_iodef(): unwanted field retrieved: id=%d",
		      val->fieldId);
	}
	if (ret == -1) {
	    break; /* the for-loop */
	}
    }
    FreeARFieldValueList(&fieldList, FALSE);
    return ret;
}

/* Function to set IODEF-Document-Address entry on ARS server */
static int setentry_iodef_address(ARControlStruct *control, ih_tree_t *tree)
{
    ARNameType		schema;
    AREntryIdType	entryId;
    ARStatusList     	status = {0, NULL};
    ARFieldValueList	fieldList;
    int			i;

    /* Set schema and entry ID */
    strcpy(schema, "CERT: IODEF-Document-Address");

    ARS_DEBUG("setentry_iodef_address():\n\tschema=%s", schema);

    fieldList.fieldValueList =
	(ARFieldValueStruct *) calloc(ADDRESS_IDS, 
				      sizeof(ARFieldValueStruct));
    if (fieldList.fieldValueList == NULL) {
	ARS_DEBUG("setentry_iodef_address(): malloc() failed");
	return -1;
    }

    i = 0;

    /* ADDRESS_Address */
        if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/IncidentData/EventData/System/Node/Address/address",
			 ADDRESS_Address,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* ADDRESS_System_category */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident/IncidentData/EventData/System", 
			  "category",
			  system_category_strs,
			  NMEMB(system_category_strs),
			  ADDRESS_System_category,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* ADDRESS_Address_category */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident/IncidentData/EventData/System/Node/Address", 
			  "category",
			  address_category_strs,
			  NMEMB(address_category_strs),
			  ADDRESS_Address_category,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* ADDRESS_IODEF_DocumentID */

    /* XXX Arne: not yet */

    fieldList.numItems = i;

    /* Get entry from the ARS server */
    if (ARCreateEntry(control, schema,  &fieldList, 
		   entryId, &status) >= AR_RETURN_ERROR) {
	ARS_DEBUG("setentry_iodef_address(): ARSetEntry failed");
	if (ars_debug == TRUE) {
	    PrintARStatusList(stderr, &status);
	}
	FreeARFieldValueList(&fieldList, FALSE);
	FreeARStatusList(&status, FALSE);
	return -1;
    }
    FreeARFieldValueList(&fieldList, FALSE);
    FreeARStatusList(&status, FALSE);
    ARS_DEBUG("setentry_iodef_address(): ARCreateEntry(): entryID=%s", entryId);
    return 0;
}

/* Function to get IODEF-Document entry from ARS server */
static int getentry_iodef(ARControlStruct *control, ih_tree_t *tree, 
			  char *entryid)
{
    ARNameType		schema;
    AREntryIdType	entryId;
    ARStatusList     	status = {0, NULL};
    AREntryIdList	entryIdList;
    ARInternalIdList	idList;
    ARFieldValueList	fieldList;
    ARInternalId	ids[DOCUMENT_IDS];
    int 		i, ret;

    /* Set schema and entry ID */
    strcpy(schema, "CERT: IODEF-Document");
    strncpy(entryId, entryid, AR_MAX_ENTRYID_SIZE);
    entryId[AR_MAX_ENTRYID_SIZE] = '\0'; /* to be safe */
    entryIdList.numItems = 1;
    entryIdList.entryIdList = &entryId;

    ARS_DEBUG("getentry_iodef():\n\tschema=%s\n\tentryId=%s",
	      schema, entryId);

    i = 0;
    ids[i++] = DOCUMENT_purpose;
    ids[i++] = DOCUMENT_restriction;
    ids[i++] = DOCUMENT_Message_type;
    ids[i++] = DOCUMENT_AlternativeIDs;
    ids[i++] = DOCUMENT_Description;
    ids[i++] = DOCUMENT_Impact_completion;
    ids[i++] = DOCUMENT_Expectation_priority;
    ids[i++] = DOCUMENT_Expectation_Description;
    ids[i++] = DOCUMENT_Impact_type;
    ids[i++] = DOCUMENT_Impact;
    ids[i++] = DOCUMENT_Method_Description;
    ids[i++] = DOCUMENT_Type;
    ids[i++] = DOCUMENT_Role;
    ids[i++] = DOCUMENT_Record_RecordData_Item;
    ids[i++] = DOCUMENT_name;
    ids[i++] = DOCUMENT_Email;
    ids[i++] = DOCUMENT_Telephone;
    ids[i++] = DOCUMENT_Timezone;

    idList.numItems = i;
    idList.internalIdList = ids;

    /* Get entry from the ARS server */
    if (ARGetEntry(control, schema, &entryIdList, &idList, 
		   &fieldList, &status) >= AR_RETURN_ERROR) {
	ARS_DEBUG("getentry_iodef(): ARGetEntry failed");
	if (ars_debug == TRUE) {
	    PrintARStatusList(stderr, &status);
	}
	FreeARStatusList(&status, FALSE);
	return -1;
    }
    FreeARStatusList(&status, FALSE);
    ARS_DEBUG("getentry_iodef(): got %d IDs", fieldList.numItems);

    /* Fill in values in the XML tree */
    (void) tree_walk(tree, "/IODEF-Document");
    (void) tree_setattr("version", "1.0");
    (void) tree_walk(tree, "/IODEF-Document/Incident/AdditionalData");
    (void) tree_setval(ECSIRT_NET_IODEF_PROFILE_V10);
    (void) tree_setattr("type", "string");
    (void) tree_setattr("meaning", "eCSIRT.net IODEF Profile Version 1.0");

    ARS_DEBUG("getentry_iodef(): ignoring NULL field IDs");
    ret = 0;
    for (i = 0; i < fieldList.numItems; i++) {
	ARFieldValueStruct *val = &fieldList.fieldValueList[i];

	if (val->value.dataType == AR_DATA_TYPE_NULL) {
	    continue; /* continue with the next value for for-loop */
	}

	ARS_DEBUG("getentry_iodef(): got fieldId=%d, type=%d",
		  val->fieldId, val->value.dataType);

	switch (val->fieldId) {
	case  DOCUMENT_purpose:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident",
				    "purpose", purpose_strs,
				    NMEMB(purpose_strs),
				    val);
	    break;

	case  DOCUMENT_restriction:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident",
				    "restriction", restriction_strs,
				    NMEMB(restriction_strs),
				    val);
	    break;

	case  DOCUMENT_Message_type:
	    /* Do nothing here */
	    break;

	case  DOCUMENT_AlternativeIDs:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/AlternativeID/IncidentID",
				   val);
	    break;

	case  DOCUMENT_Description:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/IncidentData/Description",
				   val);
	    break;

	case  DOCUMENT_Impact_completion:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident/IncidentData/Assessment/Impact",
				    "completion", completion_strs,
				    NMEMB(completion_strs),
				    val);
	    break;

	case  DOCUMENT_Expectation_priority:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident/IncidentData/Expectation",
				    "priority", priority_strs,
				    NMEMB(priority_strs),
				    val);
	    break;

	case  DOCUMENT_Expectation_Description:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/IncidentData/Expectation/Description",
				   val);
	    break;

	case  DOCUMENT_Impact_type:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident/IncidentData/Assessment/Impact",
				    "type", impact_type_strs,
				    NMEMB(impact_type_strs),
				    val);
	    break;

	case  DOCUMENT_Impact:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/IncidentData/Assessment/Impact",
				   val);
	    break;

	case  DOCUMENT_Method_Description:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/IncidentData/Method/Description",
				   val);
	    break;

	case  DOCUMENT_Type:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident/IncidentData/Contact",
				    "type", contact_type_strs,
				    NMEMB(contact_type_strs),
				    val);
	    break;

	case  DOCUMENT_Role:
	    ret = tree_setattr_enum(tree,
				    "/IODEF-Document/Incident/IncidentData/Contact",
				    "role", contact_role_strs,
				    NMEMB(contact_role_strs),
				    val);
	    break;

	case  DOCUMENT_Record_RecordData_Item:
	    /* Nothing yet here */
	    break;

	case  DOCUMENT_name:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/IncidentData/Contact/name",
				   val);
	    break;

	case  DOCUMENT_Email:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/IncidentData/Contact/Email",
				   val);
	    break;

	case  DOCUMENT_Telephone:
	    ret = tree_setval_char(tree,
				   "/IODEF-Document/Incident/IncidentData/Contact/Telephone",
				   val);
	    break;

	case  DOCUMENT_Timezone:
	    /* Do nothing here (yet)*/
	    break;

	default:
	    ARS_DEBUG("getentry_iodef_address(): unwanted field: id=%d",
		      val->fieldId);
	}
	if (ret == -1) {
	    break; /* the for-loop */
	}
    }

    /* Free allocated memory and return to caller */
    FreeARFieldValueList(&fieldList, FALSE);
    return ret;
}

/* Function to set IODEF-Document entry on ARS server */
static int setentry_iodef(ARControlStruct *control, ih_tree_t *tree)
{
    ARNameType		schema;
    AREntryIdType	entryId;
    ARStatusList     	status = {0, NULL};
    ARFieldValueList	fieldList;
    int			i;

    /* Check if XML IODEF document is eCSIRT.net IODEF Profile Version 1.0 */
    if (strcmp(ECSIRT_NET_IODEF_PROFILE_V10,
	       tree_getval(tree, "/IODEF-Document/Incident/AdditionalData"))) {
	ARS_DEBUG("setentry_iodef(): no eCSIRT.net IODEF Profile Version 1.0");
	return -1;
    }

    /* Set schema and entry ID */
    strcpy(schema, "CERT: IODEF-Document");

    ARS_DEBUG("setentry_iodef():\n\tschema=%s", schema);

    fieldList.fieldValueList =
	(ARFieldValueStruct *) calloc(DOCUMENT_IDS, 
				      sizeof(ARFieldValueStruct));
    if (fieldList.fieldValueList == NULL) {
	ARS_DEBUG("setentry_iodef(): malloc() failed");
	return -1;
    }

    i = 0;

    /* DOCUMENT_purpose */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident", 
			  "purpose",
			  purpose_strs,
			  NMEMB(purpose_strs),
			  DOCUMENT_purpose,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_restriction */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident", 
			  "restriction",
			  restriction_strs,
			  NMEMB(restriction_strs),
			  DOCUMENT_restriction,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* (NON-IODEF) DOCUMENT_Message_type */
    /*
     * FIXME, this can be done more elegantly!
     */
    fieldList.fieldValueList[i].fieldId = DOCUMENT_Message_type;
    fieldList.fieldValueList[i].value.dataType = AR_DATA_TYPE_ENUM;
    fieldList.fieldValueList[i].value.u.enumVal = (unsigned long) 0 ; /* incoming */
    i++;

    /* DOCUMENT_AlternativeIDs */
    if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/AlternativeID/IncidentID",
			 DOCUMENT_AlternativeIDs,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Description */
    if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/IncidentData/Description",
			 DOCUMENT_Description,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Impact_completion */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident/IncidentData/Assessment/Impact", 
			  "completion",
			  completion_strs,
			  NMEMB(completion_strs),
			  DOCUMENT_Impact_completion,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Expectation_priority */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident/IncidentData/Expectation", 
			  "priority",
			  priority_strs,
			  NMEMB(priority_strs),
			  DOCUMENT_Expectation_priority,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Expectation_Description */
    if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/IncidentData/Expectation/Description",
			 DOCUMENT_Expectation_Description,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Impact_type */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident/IncidentData/Assessment/Impact", 
			  "type",
			  impact_type_strs,
			  NMEMB(impact_type_strs),
			  DOCUMENT_Impact_type,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Impact */
    if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/IncidentData/Assessment/Impact",
			 DOCUMENT_Impact,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Method_Description */
    if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/IncidentData/Method/Description",
			 DOCUMENT_Method_Description,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Type */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident/IncidentData/Contact", 
			  "type",
			  contact_type_strs,
			  NMEMB(contact_type_strs),
			  DOCUMENT_Type,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Role */
    if (tree_getattr_enum(tree,
			  "/IODEF-Document/Incident/IncidentData/Contact", 
			  "role",
			  contact_role_strs,
			  NMEMB(contact_role_strs),
			  DOCUMENT_Role,
			  &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Record_RecordData_Item */
    /*
     * XXX Arne FIXME!
     */
    fieldList.fieldValueList[i].fieldId = DOCUMENT_Record_RecordData_Item;
    fieldList.fieldValueList[i].value.dataType = AR_DATA_TYPE_CHAR;
    fieldList.fieldValueList[i].value.u.charVal = "Record not yet supported" ;
    i++;

    /* DOCUMENT_name */
    if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/IncidentData/Contact/name",
			 DOCUMENT_name,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Email */
    if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/IncidentData/Contact/Email",
			 DOCUMENT_Email,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Telephone */
    if (tree_getval_char(tree,
			 "/IODEF-Document/Incident/IncidentData/Contact/Telephone",
			 DOCUMENT_Telephone,
			 &fieldList.fieldValueList[i]) >= 0) {
	i++;
    }

    /* DOCUMENT_Timezone */
    /*
     * XXX Arne FIXME!
     */
    fieldList.fieldValueList[i].fieldId = DOCUMENT_Timezone;
    fieldList.fieldValueList[i].value.dataType = AR_DATA_TYPE_CHAR;
    fieldList.fieldValueList[i].value.u.charVal = "+02:00";
    i++;

    fieldList.numItems = i;

    
    /* Get entry from the ARS server */
    if (ARCreateEntry(control, schema,  &fieldList, 
		   entryId, &status) >= AR_RETURN_ERROR) {
	ARS_DEBUG("setentry_iodef(): ARSetEntry failed");
	if (ars_debug == TRUE) {
	    PrintARStatusList(stderr, &status);
	}
	FreeARFieldValueList(&fieldList, FALSE);
	FreeARStatusList(&status, FALSE);
	return -1;
    }
    FreeARFieldValueList(&fieldList, FALSE);
    FreeARStatusList(&status, FALSE);
    ARS_DEBUG("setentry_iodef(): ARCreateEntry(): entryID=%s", entryId);
    return 0;
}

/* **************************************************************
 * IHSH ARS Remedy IODEF functions
 * ************************************************************** */

/* Function to get entry from ARS and insert it into IODEF XML tree */
int ihsh_ars_getentry_iodef(ARControlStruct *control, ih_tree_t *tree, 
			    char *entryid)
{
    int ret;

    /* Testing for now with only the IODEF-Address schema */
    (void) getentry_iodef_address; /* to fool the compiler */
    ret = getentry_iodef(control, tree, entryid);

    return ret;
}

/* Function to set IODEF XML tree contents into ARS */
int ihsh_ars_setentry_iodef(ARControlStruct *control, ih_tree_t *tree)
{
    int ret;

    /* Testing for now with only the IODEF-Address schema */
    (void) setentry_iodef_address; /* to fool the compiler */
    ret = setentry_iodef(control, tree);
    return ret;
}
