/*                               -*- Mode: C -*- 
 * 
 * ihsh_ars_misc.c -- IHSH ARS miscellaneous functions
 * 
 * Copyright (c) 2003 Arne Helme. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Arne Helme.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: ihsh_ars_misc.c,v 1.1 2003/10/22 08:16:20 arne Exp $
 * 
 */

#include <stdarg.h>
#include <stdio.h>

#include <ar.h>
#include <arstruct.h>
#include <arfree.h>
#include <arextern.h>

#include <ih/tree.h>

#include "ihsh.h"
#include "ihsh_ars_misc.h"

/* Boolean variable to toggle debug output */
int ars_debug = FALSE;


/* Function to print ARS debug information */
void ARS_DEBUG(char *fmt, ...)
{
    va_list args;

    if (ars_debug == TRUE) {
	va_start(args, fmt);
	fprintf(stderr, "%s: ARS: ", progname);
	vfprintf(stderr, fmt, args);
	fprintf(stderr, "\n");
	va_end(args);
    }
}

/* Function to print ARS status list to stream (adapted from ARS Driver) */
void PrintARStatusList(FILE *fp, ARStatusList *value)
{
    int             i;
    ARStatusStruct *tempPtr;
    char           *tempPtr2;

    fprintf(fp, "Status List : %u items\n", value->numItems);
    if (value->numItems != 0) {
	tempPtr = value->statusList;
	for (i = 0; i < (int) value->numItems; i++) {
	    switch (tempPtr->messageType) {
            case AR_RETURN_OK:
		tempPtr2 = "NOTE";
		break;
            case AR_RETURN_WARNING:
		tempPtr2 = "WARNING";
		break;
            case AR_RETURN_ERROR:
		tempPtr2 = "ERROR";
		break;
            default:
		tempPtr2 = "<unknown type>";
		break;
	    }
	    fprintf(fp,
	       "Status Struct:\n  Message type: %s\n  Message number: %ld\n",
		tempPtr2, tempPtr->messageNum);
	    if (tempPtr->messageText == NULL)
		fprintf(fp, "  Message: \n");
	    else
		fprintf(fp, "  Message: %s\n", tempPtr->messageText);
	    if (tempPtr->appendedText != NULL && tempPtr->appendedText[0])
		fprintf(fp, "  Appended: %s\n", tempPtr->appendedText);
	    tempPtr++;
	}
    }
}

/* Local unction to print ARS field value struct (adapted from ARS driver */
void PrintARFieldValueStruct(FILE *fp, ARFieldValueStruct *value)
{
   fprintf(fp, "Field Value Struct:\n");
   fprintf(fp, "   Field Id : %ld\n", value->fieldId);
   if ((value->fieldId == AR_CORE_STATUS_HISTORY) &&
       (value->value.dataType == AR_DATA_TYPE_CHAR)) {
       fprintf(fp, "   Value: %s\n", value->value.u.charVal);
   } else {
       switch (value->value.dataType) {
       case AR_DATA_TYPE_CHAR:
	   fprintf(fp, "   Value: %s\n", value->value.u.charVal);
	   break;
       case AR_DATA_TYPE_INTEGER: 
	   fprintf(fp, "   Value: %ld\n", value->value.u.intVal);
	   break;
       default:
	   fprintf(fp, "   Value: <other>\n");
       }
   }
}

/* Function to print ARS field value list (adapted from ARS driver */
void PrintARFieldValueList(FILE *fp, ARFieldValueList *value)
{
   unsigned int         i;       /* working index */
   ARFieldValueStruct  *tempPtr; /* working pointer */

   fprintf(fp, "Field Value List : %u items\n", value->numItems);
   if (value->numItems != 0) {
      tempPtr = value->fieldValueList;
      for (i = 0; i < value->numItems; i++) {
         PrintARFieldValueStruct(fp, tempPtr);
         tempPtr++;
      }
   }
}
