/*                               -*- Mode: C -*- 
 * 
 * ihsh_misc.c -- Miscellaneous IHSH functions
 * 
 * Copyright (c) 2003 Arne Helme. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Arne Helme.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: ihsh_misc.c,v 1.1 2003/10/22 08:16:20 arne Exp $
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ihsh_compat.h"
#include "ihsh_misc.h"


/* Function to read a line including newline '\n' character from stream.  
   The caller is responsible for freeing the allocated memory */
char *ihsh_getline(FILE *fp)
{
    char *buf = NULL;
    char *lbuf = NULL;
    size_t len = 0;

    buf = fgetln(fp, &len);
    if (buf != NULL) {
	if ((lbuf = (char *) malloc(len + 1)) == NULL) {
	    return NULL;
	}
	(void) memcpy(lbuf, buf, len);
	lbuf[len] = '\0';
	buf = lbuf;
    }
    return len >= 0 ? buf : NULL;
}

/* Function to read multiple lines from stream. The caller is responsible for 
   freeing the allocated memory */
char *ihsh_getlines(FILE *fp)
{
    char *buf, *lbuf;

    buf = ""; /* empty string */
    while ((lbuf = ihsh_getline(fp)) != NULL) {
	/* a single dot on a line means end of input */
	if (lbuf[0] == '.' && (lbuf[1] == '\0' || lbuf[1] == '\n')) {
	    free(lbuf);
	    break;
	}
	buf = (char *) realloc(buf, strlen(buf) + strlen(lbuf) + 1);
	if (buf == NULL) {
	    free(lbuf);
	    return NULL;
	}
	(void) strcpy(&buf[strlen(buf)], lbuf);
	free(lbuf);
    }
    /* Remove trailing newline character, if any */
    if (buf[strlen(buf) - 1] == '\n') {
	buf[strlen(buf) - 1] = '\0';
    }
    return buf;
}
