/*                               -*- Mode: C -*- 
 * 
 * ihsh_xml.c -- IHSH XML specific functions
 * 
 * Copyright (c) 2003 Arne Helme. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Arne Helme.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: ihsh_xml.c,v 1.3 2003/10/22 11:58:39 arne Exp $
 * 
 */

#include <stdio.h>

#include <ih/tree.h>
#include <ih/tree_xml.h>
#include <ih/xml.h>

#include "ihsh.h"
#include "ihsh_xml.h"

/* Maximum size of XML document we van handle */
#ifndef DOC_CHUNK_SIZE
#define DOC_CHUNK_SIZE 50000
#endif

/* Function to read XML document from a stream */
int ihsh_xml_fread(ih_tree_t **tree_p, FILE *fp, ualloc_t *ua)
{
    char 			*doc;
    ih_err_t 			result;
    ih_data_size_t 		len;
    ih_tree_schema_type_t	tree_type;
    ih_asciz_t 			xml_parse_error;
    
    doc = malloc(DOC_CHUNK_SIZE + 1);
    if (doc == NULL) {
	return -1;
    }
    memset(doc, 0, DOC_CHUNK_SIZE);
    fread(doc, DOC_CHUNK_SIZE - 1, 1, fp);  /* XXX Arne: may need a loop !*/

    result = ih_xml_identify(doc, &tree_type);
    if (result != IH_E_NONE) {
	fprintf(stderr, "Unknown XML document type\n");
	return -1;
    } else if (debug) {
	fprintf(stderr, "XML Document DTD type: %s\n", 
		ih_tree_type2str(tree_type));
    }
    
    len = strlen(doc);
    if ((result = ih_xml_parse(doc, len, 
			       tree_type, tree_p, ua, 
			       &xml_parse_error)) != IH_E_NONE) {
	printf("Error: xml_parse: result = %d (%s)\n", 
	       result, ih_err_code2str(result));
	if ((result == IH_E_BAD_XML) && xml_parse_error) {
	    fprintf(stderr, "Error: xml_parse: expat error: %s\n", 
		    xml_parse_error);
	    ualloc_free(ua, xml_parse_error);
	}
    } else if (debug) {
	printf("XML parse: OK\n");
    }
    free(doc);
    return 0;
}

/* Function to write format and write XML tree to a stream */
int ihsh_xml_fwrite(ih_tree_t *tree, FILE *fp)
{
    air_xbuf_t 	*buf;
    ih_err_t 	result;
    
    buf = air_xbuf_alloc(NULL, DOC_CHUNK_SIZE);
    if (buf == NULL) {
	return -1;
    }
    result = ih_tree_xml(tree, buf, NULL);
    if (result != IH_E_NONE) {
	air_xbuf_free(buf);
	return -1;
    }
    fprintf(fp, "%s", air_xbuf_string(buf, NULL));
    air_xbuf_free(buf);
    return 0;
}
