/*                               -*- Mode: C -*- 
 * 
 * ih_test.c -- Test functions for LibIH & IHSH
 * 
 * Copyright (c) 2003 Arne Helme. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Arne Helme.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: ih_test.c,v 1.1 2003/10/21 11:14:44 arne Exp $
 * 
 */

#include <stdio.h>

/* #include <expat.h> */

#include <ih/tree.h>
#include <ih/tree_xml.h>
/* #include <ih/sanitize.h> */

typedef enum {FAILED, SUCCESS} res_t;

ualloc_t	*ua;

static void tree_fwrite(ih_tree_t *tree, FILE *fp)
{
    ih_err_t	result;
    air_xbuf_t	*xbuf;

    xbuf = air_xbuf_alloc(ua, 4192);
    if (xbuf == NULL) {
	printf("Error: could not allocate 4192 byte xbuf\n");
	exit(EXIT_FAILURE);
    }
    result = ih_tree_xml(tree, xbuf, NULL);
    if (result != IH_E_NONE) {
	return;
    }
    printf("=========================\n%s\n", air_xbuf_bytes(xbuf, NULL));
    air_xbuf_free(xbuf);
}

static res_t test_insert(void)
{
    res_t		res;
    ih_err_t		result;
    ih_tree_t		*tree;
    ih_data_size_t	len;

    res = SUCCESS;
    tree = ih_tree_create(ua, IH_TREE_T_IODEF);
    if (tree == NULL) {
	fprintf(stderr, "could not create initial XML tree\n");
	exit(EXIT_FAILURE);
    }
    
    len = 13;
    result = ih_tree_set2(tree, 
			 "/IODEF-Document/Incident/IncidentData/EventData/System(1)/Node/Address/address", 
			 "129.242.16.25", 
			 &len);
    if (result != IH_E_NONE) {
	printf("Error: tree_set: result = %d\n", result);
	res = FAILED;
    }

    len = 13;
    result = ih_tree_set2(tree, 
			 "/IODEF-Document/Incident/IncidentData/EventData/System(2)", 
			 "129.242.16.26", 
			 &len);
    if (result != IH_E_NONE) {
	printf("Error: tree_set: result = %d\n", result);
	res = FAILED;
    }

    len = 13;
    result = ih_tree_set2(tree, 
			 "/IODEF-Document/Incident/IncidentData/EventData/System(2)/Node/Address/address", 
			 "129.242.16.26", 
			 &len);
    if (result != IH_E_NONE) {
	printf("Error: tree_set: result = %d\n", result);
	res = FAILED;
    }

    if (res == SUCCESS) {
	tree_fwrite(tree, stdout);
    }
    if (ih_tree_destroy(tree) != IH_E_NONE ) {
	fprintf(stderr, "could destroy XML tree\n");
	exit(EXIT_FAILURE);
    }
    return res;
}

static const char *res2str(res_t res)
{
    const char *success = "SUCCESS";
    const char *failed = "FAILED";
    const char *ret;

    if (res == SUCCESS) {
	ret = success;
    } else {
	ret = failed;
    }
    return ret;
}

int main(int argc, char *argv[])
{
    ua = ualloc_create(NULL, NULL, 0, NULL, NULL, NULL);
    if (ua == NULL) {
	fprintf(stderr, "could not initialize LibIH memory pool\n");
	exit(EXIT_FAILURE);
    }

    printf("test_insert(): %s\n", res2str(test_insert()));
    exit(EXIT_SUCCESS);
}
